using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using PPCUpdater.PPCUpdaterWebService;

namespace PPCUpdater
{
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TextBox txtStatus;
		private System.Windows.Forms.Button btnCheckForUpdates;
		private System.Windows.Forms.MainMenu mainMenu1;

		private XmlDocument XmlServerFileInventory = new XmlDocument();
		private XmlDocument XmlLocalFileInventory = new XmlDocument();
		private StringBuilder sbUpdateList = new StringBuilder();

		private Updater upd;
		private clsConfig config = new clsConfig();
		
		private const string LOCALFILEINVENTORY = "FileInventory.txt";

        /// <summary>
        /// PInvoke CreateProcess in order to programatically execute the parent application
        /// </summary>
		[DllImport("coredll.Dll")]
		private static extern int CreateProcess(string strImageName, string strCmdLine, IntPtr pProcessAttributes, IntPtr pThreadAttributes , int bInheritsHandle, int dwCreationFlags, IntPtr pEnvironment, IntPtr pCurrentDir, Byte[] bArray, ProcessInfo oProc); 
		private ProcessInfo pi = new ProcessInfo();
		private class ProcessInfo
		{
			public int Process = 0; 
			public int Thread = 0; 
			public int ProcessID = 0; 
			public int ThreadID = 0; 
		}

		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.btnCheckForUpdates = new System.Windows.Forms.Button();
			// 
			// txtStatus
			// 
			this.txtStatus.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular);
			this.txtStatus.Location = new System.Drawing.Point(8, 8);
			this.txtStatus.Multiline = true;
			this.txtStatus.Size = new System.Drawing.Size(224, 224);
			this.txtStatus.Text = "";
			// 
			// btnCheckForUpdates
			// 
			this.btnCheckForUpdates.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.btnCheckForUpdates.Location = new System.Drawing.Point(48, 240);
			this.btnCheckForUpdates.Size = new System.Drawing.Size(136, 24);
			this.btnCheckForUpdates.Text = "Check For Updates";
			this.btnCheckForUpdates.Click += new System.EventHandler(this.btnCheckForUpdates_Click);
			// 
			// MainForm
			// 
			this.Controls.Add(this.btnCheckForUpdates);
			this.Controls.Add(this.txtStatus);
			this.Menu = this.mainMenu1;
			this.Text = "Updater";
			this.Load += new System.EventHandler(this.MainForm_Load);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>

		static void Main() 
		{
			Application.Run(new MainForm());
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{
		
			upd = new Updater();
		    //read the web service URL from our configuration file
			upd.Url = config.webserviceurl;
            //Uncomment the line below if you activated NTLM Authentication on the web service
			//upd.Credentials = new System.Net.NetworkCredential(config.userid,config.password,config.domain);

		}

		private void btnCheckForUpdates_Click(object sender, System.EventArgs e)
		{
			CheckForUpdates();
		}

		private void CheckForUpdates()
		{
			try
			{
				Cursor.Current = Cursors.WaitCursor;
				XmlServerFileInventory.LoadXml(upd.GetFileInventory());
				GetUpdateList();
			}	
			catch(Exception e)
			{
				MessageBox.Show(e.Message);
			}
			finally
			{
				Cursor.Current = Cursors.Default;
			}		
		}

		private void GetUpdateList()
		{
			
			sbUpdateList = new StringBuilder();
			bool GetFile = true;
			bool GetAll = false;

            //load the local xml file inventory
			if(!File.Exists(LOCALFILEINVENTORY))
			{
				XmlServerFileInventory.Save(LOCALFILEINVENTORY);
				GetAll = true;
			}

			XmlLocalFileInventory.Load(LOCALFILEINVENTORY);

            //build an xml string of files we need to bring down from the server
            foreach(XmlNode n in XmlServerFileInventory.DocumentElement.ChildNodes)
			{
				GetFile = false;
				if(File.Exists(n.Attributes.GetNamedItem("destination").Value) && !GetAll)
				{
					switch(n.Attributes.GetNamedItem("action").Value)
					{
                        //Ensure that the file timestamp is equal to the server timestamp
                        case "current":
							foreach(XmlNode l in XmlLocalFileInventory.DocumentElement.ChildNodes)
							{
								if(n.Attributes.GetNamedItem("destination").Value == l.Attributes.GetNamedItem("destination").Value &&
								   Convert.ToDateTime(l.Attributes.GetNamedItem("date").Value) < Convert.ToDateTime(n.Attributes.GetNamedItem("date").Value))
								{
									GetFile = true;
									l.Attributes.GetNamedItem("date").Value = n.Attributes.GetNamedItem("date").Value;
									break;
								}
							}
							break;
                        //always replace with the version on the server
						case "replace":
							GetFile = true;
							break;
						default:
							GetFile = false;
							break;
					}
				}
				else
				{
					GetFile = true;
				}

				if(GetFile)
				{
					sbUpdateList.Append("<file ");
					sbUpdateList.Append("name=\"");
					sbUpdateList.Append(n.Attributes.GetNamedItem("name").Value);
					sbUpdateList.Append("\" ");
					sbUpdateList.Append("date=\"");
					sbUpdateList.Append(n.Attributes.GetNamedItem("date").Value);
					sbUpdateList.Append("\" ");
					sbUpdateList.Append("destination=\"");
					sbUpdateList.Append(n.Attributes.GetNamedItem("destination").Value);
					sbUpdateList.Append("\" />");
				}
			}
			
			if(sbUpdateList.Length>0)
			{
                //if there are updates, we ask if they want to download them
                if(MessageBox.Show("Updates are available. Would you like to automatically install them?","",MessageBoxButtons.YesNo,MessageBoxIcon.Question,MessageBoxDefaultButton.Button1) == DialogResult.Yes)
				{
					DownloadAndInstall();
				}
				else
				{
					CreateProcess(config.parentappexe, "", IntPtr.Zero, IntPtr.Zero, 0, 0, IntPtr.Zero, IntPtr.Zero, new Byte[128], pi); 
					Application.Exit();
				}
			}
			else
			{
				MessageBox.Show("There are no new updates.");
				CreateProcess(config.parentappexe, "", IntPtr.Zero, IntPtr.Zero, 0, 0, IntPtr.Zero, IntPtr.Zero, new Byte[128], pi); 
				Application.Exit();
			}
		
		}
	
		private void DownloadAndInstall()
		{
			XmlDocument xDoc = new XmlDocument();
			string destination = "";
			string session = DateTime.Now.ToLongTimeString().Replace("/","").Replace(":","").Replace(" ","");
			Hashtable hsh = new Hashtable();
			bool rollback = false;

			//Terminate the xml string in preparation for submission to the webservice
			sbUpdateList.Insert(0,"<root>");
			sbUpdateList.Append("</root>");

			//get the xml string from the webservice that contains all the
			//binaries that we need
			xDoc.LoadXml(upd.GetBinaries(sbUpdateList.ToString()));

			try
			{

				//loop through the XML
				foreach(XmlNode n in xDoc.DocumentElement.ChildNodes)
				{
					destination = n.Attributes.GetNamedItem("destination").Value;
					if(File.Exists(destination))
					{
						//store the files original name and backup name in our hash
						hsh.Add(destination+"." + session,destination);
						//rename the current file, in case we need to roll back
						File.Move(destination,destination+"." + session);
					}

					//Use the GetFileFromBase64String to decode the file from
					//the base64 string that is embedded in the xml document
					txtStatus.Text += "Updated: " + GetFileFromBase64String(n.Attributes.GetNamedItem("value").Value,destination) + "\r\n";
				}
			}
			catch
			{
				rollback = true;
			}
			finally
			{
				if(rollback)
				{
					//rollback manually
					foreach(string key in hsh.Keys)
					{
						if (File.Exists(key))
						{
							File.Move(key,hsh[key].ToString());
						}
					}
					MessageBox.Show("An Unexpected error has occured while trying to update. Please perform a soft reset on the device, and try again.");
					Application.Exit();
				}
				else
				{
					//delete all backup files
					foreach(string key in hsh.Keys)
					{
						if (File.Exists(key))
						{
							File.Delete(key);
						}
					}
					//update the local inventory file
					XmlLocalFileInventory.Save(LOCALFILEINVENTORY);

					MessageBox.Show("Update is successful.");
					CreateProcess(config.parentappexe, "", IntPtr.Zero, IntPtr.Zero, 0, 0, IntPtr.Zero, IntPtr.Zero, new Byte[128], pi); 
					Application.Exit();
				}
			}
		}


		public string GetFileFromBase64String(string Base64String, string FileName)
		{
			if(!Directory.Exists(Path.GetDirectoryName(FileName)))
			{
				Directory.CreateDirectory(Path.GetDirectoryName(FileName));
			}
			byte[] newBinaryData = System.Convert.FromBase64String(Base64String);
			FileStream newFile = File.Create(FileName,newBinaryData.Length);
			newFile.Write(newBinaryData,0,newBinaryData.Length);
			newFile.Close();

			FileInfo fi = new FileInfo(FileName);
			return Path.GetFileName(FileName);
		}


	}
}
