using System;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Windows.Forms;

namespace PPCUpdater
{
	/// <summary>
	/// We read and store the application configuration variables in a text (txt) file, 
    /// so that we can easily open and modify it within the pocket pc. 
	/// </summary>
	public class clsConfig
	{
		private XmlDocument xDoc = new XmlDocument();
		private string configfile = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase), "PPCUpdaterconfig.txt") ;

		public string contractorid{get{return GetValue("deviceid");}set{SetValue("deviceid",value);}} 
		public string version{get{return GetValue("version");}set{SetValue("version",value);}} 
		public string webserviceurl{get{return GetValue("webserviceurl");}set{SetValue("webserviceurl",value);}}
		public string userid{get{return GetValue("userid");}set{SetValue("userid",value);}}
		public string password{get{return GetValue("password");}set{SetValue("password",value);}}
		public string domain{get{return GetValue("domain");}set{SetValue("domain",value);}}
		public string parentappexe{get{return GetValue("parentappexe");}set{SetValue("parentappexe",value);}} 

		public clsConfig()
		{
			try
			{
                xDoc.Load(configfile);
			}
			catch(Exception e)
			{
				MessageBox.Show("Error retrieving application variables: " + e.Message);
			}
		}

		private string GetValue(string Key)
		{
			try
			{
                return xDoc.DocumentElement.Attributes.GetNamedItem(Key).Value;
			}
			catch(Exception e)
			{
				MessageBox.Show("Error retrieving config variables: " + Key + ":" + e.Message);
				return "";
			}
		}

		private void SetValue(string Key, string Value)
		{
			try
			{
                xDoc.DocumentElement.Attributes.GetNamedItem(Key).Value = Value;
			}
			catch(Exception e)
			{
				MessageBox.Show("Error saving config variables: " + e.Message);
			}
		}

		public void Save()
		{
            xDoc.Save(configfile);
            xDoc.Load(configfile);
		}

	}
}
